#include <io.h>         // Lowlevel Input/Output
#include <stdio.h>      // Standard Input/Output
#include <string.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <errno.h>

//------------------------------------------------------------
// int fnCopyFile (LPCTSTR lpFile, LPCTSTR lpNewFile);
//------------------------------------------------------------
// Parameter
// lpFileName - Name der existierenden datei
// lpNewFile  - Name der Zieldatei
//------------------------------------------------------------
// Return-Werte
//    0 = Operation war erfolgreich
//------------------------------------------------------------
// -10 = Quelldatei ist nicht zu ffnen  (ERROR NOT SPECIFIED)
// -11 = Quelldatei ist nicht zu ffnen  (ACCESS DENIED)
// -12 = Quelldatei ist nicht zu ffnen  (INVALID ACCESS CODE)
// -13 = Quelldatei ist nicht zu ffnen  (TOO MANY OPEN FILES)
// -14 = Quelldatei ist nicht zu ffnen  (NO SUCH FILE OR DIR)
//------------------------------------------------------------
// -20 = Zieldatei ist nicht zu ffnen  (ERROR NOT SPECIFIED)
// -21 = Zieldatei ist nicht zu ffnen  (ACCESS DENIED)
// -22 = Zieldatei ist nicht zu ffnen  (INVALID ACCESS CODE)
// -23 = Zieldatei ist nicht zu ffnen  (TOO MANY OPEN FILES)
// -24 = Zieldatei ist nicht zu ffnen  (NO SUCH FILE OR DIR)
//------------------------------------------------------------
// -30 = Buffer kann nicht angelegt werden
//------------------------------------------------------------
// -40 = Lesezugriff verweigert
// -41 = Bad Filenumber
//------------------------------------------------------------
// -50 = Schreibzugriff verweigert
// -51 = Bad Filenumber
//------------------------------------------------------------
// -60 = allgemeiner Kopierfehler
//------------------------------------------------------------
// -70 = Quell und Zieldatei sind identisch
//------------------------------------------------------------
int fnCopyFile (char *lpFile, char *lpNewFile)
{
   #define MYBUFFERSIZE 32000
   extern int errno;
   int   nRHandle, nWHandle; // Handle Quell- und Zieldatei
   int   nBytes = 1;         // zurckgemeldete Buffergre
   int   nResult;            // Anzahl geschriebene Zeichen
   int   nOk  = 1;           // Schreiboperation ok
   void *buff = NULL;        // Datenpuffer
   int   nError = 0;         // Fehlernummer

   if (!stricmp (lpFile, lpNewFile))
      return (-70);

   //---------------------------------------------------------
   // Quell- und Zieldatei ffnen
   //---------------------------------------------------------
   nRHandle = open (lpFile, O_RDONLY|O_BINARY, 
                    S_IWRITE|S_IREAD);

   if(-1 == nRHandle)
   {
      switch (errno)
       {
         default:      nError = -10; break;
         case EACCES:  nError = -11; break;
         case EINVACC: nError = -12; break;
         case EMFILE:  nError = -13; break;
         case ENOENT:  nError = -14; break;
      }
      return (nError);
   }

   nWHandle = open (lpNewFile, O_CREAT|O_WRONLY|O_BINARY,
                    S_IWRITE|S_IREAD);

   if(-1 == nWHandle)
   {
      close (nRHandle);
      switch (errno)
      {
         default:      nError = -20; break;
         case EACCES:  nError = -21; break;
         case EINVACC: nError = -22; break;
         case EMFILE:  nError = -23; break;
         case ENOENT:  nError = -24; break;
      }
   }

   //---------------------------------------------------------
   // Beide Dateien sind offen, jetzt Bufferbereich anlegen
   // Wenn Buffer nicht angelegt werden kann, dann return -30
   //---------------------------------------------------------
   buff = new unsigned char [MYBUFFERSIZE];

   if (!buff)
   {
      close (nRHandle);
      close (nWHandle);
      return (-30);
   }

   //---------------------------------------------------------
   // Beide Dateien sind offen und Puffer existiert
   // Kopierprozess kann beginnen
   //---------------------------------------------------------
   while ((nBytes) && (nOk) && (!nError))
   {
      nBytes  = read  (nRHandle, buff, MYBUFFERSIZE);
      if (nBytes == -1)
      {
         switch (errno)
         {
            case EACCES: nError = -40; break; 
            case EBADF:  nError = -41; break;
         }
         nOk = 0;
      }
      nResult = write (nWHandle, buff, nBytes);
      if (nResult == -1)
      {
         switch (errno)
         {
            case EACCES: nError = -50; break;
            case EBADF:  nError = -51; break;
         }
         nOk = 0;
      }

      if (nBytes != nResult)
      {
         nOk = 0;
         if (!nError)
            nError = -60;
      }
   }

   //---------------------------------------------------------
   // Alles aufrumen
   //---------------------------------------------------------
   close (nRHandle);
   close (nWHandle);
   delete [] buff;

   //---------------------------------------------------------
   // Fehler beim Kopieren, dann Zielfile wieder entfernen
   //---------------------------------------------------------
   if (nError < 0)
   {
      remove (lpNewFile);
      return (nError);
   }

   //---------------------------------------------------------
   // Alles ok
   //---------------------------------------------------------
   return (0);
   #undef MYBUFFERSIZE
}

//------------------------------------------------------------
// int fnMoveFile (LPCTSTR lpFile, LPCTSTR lpNewFile);
//------------------------------------------------------------
// Parameter
// lpFile     - Name der zu kopierenden Datei
// lpNewFile  - Name der Zieldatei
//------------------------------------------------------------
// Return-Werte
//    0 = Operation war erfolgreich
//------------------------------------------------------------
// -10 = Quelldatei ist nicht zu ffnen  (ERROR NOT SPECIFIED)
// -11 = Quelldatei ist nicht zu ffnen  (ACCESS DENIED)
// -12 = Quelldatei ist nicht zu ffnen  (INVALID ACCESS CODE)
// -13 = Quelldatei ist nicht zu ffnen  (TOO MANY OPEN FILES)
// -14 = Quelldatei ist nicht zu ffnen  (NO SUCH FILE OR DIR)
//------------------------------------------------------------
// -20 = Zieldatei ist nicht zu ffnen  (ERROR NOT SPECIFIED)
// -21 = Zieldatei ist nicht zu ffnen  (ACCESS DENIED)
// -22 = Zieldatei ist nicht zu ffnen  (INVALID ACCESS CODE)
// -23 = Zieldatei ist nicht zu ffnen  (TOO MANY OPEN FILES)
// -24 = Zieldatei ist nicht zu ffnen  (NO SUCH FILE OR DIR)
//------------------------------------------------------------
// -30 = Buffer kann nicht angelegt werden
//------------------------------------------------------------
// -40 = Lesezugriff verweigert
// -41 = Bad Filenumber
//------------------------------------------------------------
// -50 = Schreibzugriff verweigert
// -51 = Bad Filenumber
//------------------------------------------------------------
// -60 = allgemeiner Kopierfehler
//------------------------------------------------------------
// -70 = Quell und Zieldatei sind identisch
//------------------------------------------------------------
int fnMoveFile (char *lpFile, char *lpNewFile)
{
   int nResult = fnCopyFile (lpFile, lpNewFile);

   if (!nResult)
      remove (lpFile);

   return (nResult);
}
