//============================================================
// Programm DYNSTR.CPP
//============================================================

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include "dynstr.h"

#define ENDSTR '\0'

//============================================================
// Constructor der dynamischen Stringklasse
//============================================================
dynstr::dynstr ()
{
   //---------------------------------------------------------
   // Alle Ptr sollten stets einen gltigen Wert haben
   //---------------------------------------------------------
   ptr = NULL;
}

//============================================================
// Copy-Constructor der dynamischen Stringklasse
//============================================================
dynstr::dynstr (dynstr& x)
{
   set (x.ptr);
}

//============================================================
// Destructor der dynamischen Stringklasse
//============================================================
dynstr::~dynstr ()
{
   //---------------------------------------------------------
   // Freigabe ist nur dann sinnvoll, wenn auch was drinsteht!
   // VORSICHT, wenn Null-Pointer freigegeben wird besteht
   // Absturzgefahr !
   //---------------------------------------------------------
   if (ptr) delete ptr;
}

//============================================================
// String in Kleinschreibung umwandeln
//============================================================
void dynstr::lower (void)
{
   char *help = ptr;

   //---------------------------------------------------------
   // Wenn keine Daten vorhanden, kann man nichts tun
   //---------------------------------------------------------
   if (!ptr) return;

   //---------------------------------------------------------
   // Wenn Daten vorhanden sind, dann umsetzen (insbesondere 
   // deutsche Sonderzeichen)
   //---------------------------------------------------------
   for (; *help; help++)
   {
      if ((*help > 64) && (*help < 91)) *help += (char)32;
      else switch (*help)
      {
         case '' : *help = ''; break;
         case '' : *help = ''; break;
         case '' : *help = ''; break;
      }
   }
}

//============================================================
// String in Groschreibung umwandeln
//============================================================
void dynstr::upper (void)
{
   char *help = ptr;

   //---------------------------------------------------------
   // Wenn keine Daten vorhanden, kann man nichts tun
   //---------------------------------------------------------
   if (!ptr) return;

   //---------------------------------------------------------
   // Wenn Daten vorhanden sind, dann umsetzen (insbesondere
   // deutsche Sonderzeichen)
   //---------------------------------------------------------
   for (; *help; help++)
   {
      if ((*help > 96) && (*help < 123)) *help -= (char)32;
      else switch (*help)
      {
         case '' : *help = ''; break;
         case '' : *help = ''; break;
         case '' : *help = ''; break;
      }
   }
}

//============================================================
// Leerzeichen links abschneiden
//============================================================
char *dynstr::ltrim (void)
{
   char *help = ptr;
   char *newp = NULL;
   char *nptr = NULL;

   //---------------------------------------------------------
   // Wenn keine Daten vorhanden, kann man nichts tun
   //---------------------------------------------------------
   if (!ptr) return (NULL);

   //---------------------------------------------------------
   // Wenn der ASCII-Wert des aktuellen Zeichens kleiner ist
   // als 33, dann ist es ein Whitespace-Zeichen und soll
   // abgeschnitten werden. Hier wird nach dem ersten nicht-
   // Whitespace gesucht
   //---------------------------------------------------------
   while ((*help < 33) && (*help != '\0')) help++;

   //---------------------------------------------------------
   // Wenn help jetzt auch '\0'-Zeichen steht, dann enthlt 
   // die Zeichenkette nur Whitespace-Zeichen und wird
   // zurckgesetzt
   //---------------------------------------------------------
   if (*help == ENDSTR)
   {
      ptr[0] = ENDSTR;
      return (ptr);
   }

   //---------------------------------------------------------
   // Wenn die Zeichenkette verwertbare Zeichen enthlt, dann 
   // neuen Speicher fr die Restlnge (ohne fhrende 
   // Whitespaces) anfordern
   //---------------------------------------------------------
   nptr = new char [len (help) + 1];

   //---------------------------------------------------------
   // Kein Speicher verfgbar? dann alten String zurck
   //---------------------------------------------------------
   if (!nptr) return (ptr);

   //---------------------------------------------------------
   // Wenn Speicher verfgbar, dann Zeichenweise kopieren und 
   // mit String-Ende-Zeichen abschlieen
   //---------------------------------------------------------
   for (newp = nptr;*help;help++,newp++) *newp = *help;
   *newp = 0;

   //---------------------------------------------------------
   // Nach dem kopieren den alten Speicher wieder freigeben 
   // und den internen Stringpointer auf den neuen
   // Speicherbereich zeigen lassen. Zum Schlu  noch den 
   // Stringpointer zurckgeben
   //---------------------------------------------------------
   delete ptr;
   ptr = nptr;
   return (ptr);
}

//============================================================
// Leerzeichen rechts abschneiden
//============================================================
char *dynstr::rtrim (void)
{
   char *help = ptr;
   char *nptr = NULL;
   char *newp = NULL;
   int i = len (ptr);

   //---------------------------------------------------------
   // Wenn keine Daten vorhanden, kann man nichts tun
   //---------------------------------------------------------
   if ((!ptr) || (i < 0)) return (NULL);
   while ((help [i] < 33) && (i >= 0)) i--;
   help[i+1] = 0;

   //---------------------------------------------------------
   // Wenn die Zeichenkette verwertbare Zeichen enthlt, dann 
   // neuen Speicher fr die Restlnge (ohne folgende 
   // Whitespaces) anfordern
   //---------------------------------------------------------
   nptr = new char [len (ptr) + 1];

   //---------------------------------------------------------
   // Kein Speicher verfgba? dann alten String zurck
   //---------------------------------------------------------
   if (!nptr) return (ptr);

   //---------------------------------------------------------
   // Wenn Speicher verfgbar, dann Zeichenweise kopieren und 
   // mit String-Ende-Zeichen abschlieen
   //---------------------------------------------------------
   for (newp = nptr, help = ptr; *help; help++, newp++) 
      *newp = *help;
   *newp = 0;

   //---------------------------------------------------------
   // Nach dem kopieren den alten Speicher wieder freigeben 
   // und den internen Stringpointer auf den neuen 
   // Speicherbereich zeigen
   // lassen. Zum Schlu  noch den Stringpointer zurckgeben
   //---------------------------------------------------------
   delete ptr;
   ptr = nptr;
   return (ptr);
}

//============================================================
// Fhrende und abschlieende Leerzeichen wegschneiden
//============================================================
char *dynstr::alltrim (void)
{
   rtrim();
   ltrim();
   return (ptr);
}

//============================================================
// Setzen des Strings
//============================================================
void dynstr::set (char *toset)
{
   int size = len (toset) + 1;
   char *help = NULL;

   //---------------------------------------------------------
   // Wenn schon Daten vorhanden sind, mssen diese zunchst
   // freigegeben werden
   //---------------------------------------------------------
   if (ptr) delete ptr;

   //---------------------------------------------------------
   // neuen Speicher in der bentigten Gre besorgen
   //---------------------------------------------------------
   ptr = new char [size];

   //---------------------------------------------------------
   // Wenn wir den Speicher bekommen haben, aus Parameter
   // herberkopieren
   //---------------------------------------------------------
   if (!ptr) return;
   for (help=ptr;*toset;help++,toset++) *help = *toset;

   //---------------------------------------------------------
   // und zum Schlu das Stringende setzen
   //---------------------------------------------------------
   ptr [size-1] = 0;
}

//============================================================
// Anhngen an den gespeicherten String
//============================================================
void dynstr::concat (char *toadd)
{
   int  size;
   char *nptr = NULL, *nhelp = NULL, *ohelp = NULL;

   //---------------------------------------------------------
   // Wenn keine Daten vorhanden, kann man nichts tun
   //---------------------------------------------------------
   if (!ptr) return;

   //---------------------------------------------------------
   // der benigte Speicherplatz ist die Lnge des alten 
   // Strings plus die Lnge des neuen Strings
   //---------------------------------------------------------
   size = len (ptr) + len (toadd) + 1;

   //---------------------------------------------------------
   // Platz fr den zusammengesetzten String besorgen
   //---------------------------------------------------------
   nptr = new char [size];

   //---------------------------------------------------------
   // Wenn nicht gengend Speicher vorhanden ist, Adios
   //---------------------------------------------------------
   if (!nptr) return;

   //---------------------------------------------------------
   // zunchst den alten String kopieren, dann den neuen Teil
   // dahinter und zuletzt mit String-Ende-Symbol  abschlieen
   //---------------------------------------------------------
   for (nhelp=nptr,ohelp=ptr;*(ohelp);nhelp++,ohelp++)
       *nhelp=*ohelp;
   for (; *(toadd); nhelp++, toadd++) *nhelp = *toadd;
   *nhelp = 0;

   //---------------------------------------------------------
   // Nun noch den alten String freigeben und den internen
   // Stringpointer auf den neuen Speicherbereich zeigen
   // lassen
   //---------------------------------------------------------
   delete ptr;
   ptr = nptr;
}

//============================================================
// Lnge eines Strings ermitteln
//============================================================
int dynstr::len (char *tocount) const
{
   char *help = tocount;

   //---------------------------------------------------------
   // Startadresse minus Endadresse ist natrlich die Gre in
   // Bytes (ohne String-Ende-Symbol)
   //---------------------------------------------------------
   for (; *help; help++);
   return ((int)(help-tocount));
}

//============================================================
// Zuweisungsoperator berladen, da sonst der Pointer kopiert 
// wird und nicht der Inhalt des Objekts
//============================================================
char *dynstr::operator= (dynstr & tocopy)
{
   set (tocopy.ptr);
   return (ptr);
}

//============================================================
// Zuweisungsoperator berladen, da sonst der Pointer kopiert 
// wird und nicht der Inhalt des Objekts
//============================================================
char *dynstr::operator= (char *tocopy)
{
   set (tocopy);
   return (ptr);
}

//============================================================
// Vergleichsoperator berladen, da sonst die Stringadressen
// verglichen werden, nicht die Objektinhalte
//============================================================
int dynstr::operator== (dynstr & tocheck)
{
   int nlen = len (ptr);
   if (nlen != len (tocheck.ptr)) return (0);

   while (nlen >= 0)
   {
      if (tocheck.ptr[nlen] != ptr[nlen]) return (0);
      nlen--;
   }
   return (1);
}

//============================================================
// Vergleichsoperator berladen, da sonst die Stringadressen
// verglichen werden, nicht die Objektinhalte
//============================================================
int dynstr::operator== (char *tocheck)
{
   int nlen = len (ptr);
   if (nlen != len (tocheck)) return (0);

   while (nlen >= 0)
   {
      if (tocheck[nlen] != ptr[nlen]) return (0);
      nlen--;
   }
   return (1);
}

//============================================================
// Beispielprogramm
//============================================================

int main (void)
{
   dynstr x;
   dynstr y;

   x.set (" Alltrim Test ");
   y.set (" Alltrim Test ");
   cout << "SET      [" <<x.get () << "]" << endl;

   cout << "Vergleich : " << (x == y) << endl;

   x.alltrim ();
   cout << "Vergleich : " << (x == y) << endl;
   cout << "Vergleich : " << (x=="Alltrim Test")<< endl;
   cout << "ALLTRIM  [" <<x.get () << "]" << endl;
   cout << (y = x) << endl;
   return (0);
}
